#!/usr/bin/env python
# -*- coding: utf-8 -*-

#WRITTEN BY DENE BOWDALO

###------------------------------------------------------------------------------------###
###IMPORT MODULES
###------------------------------------------------------------------------------------###

import crossnetwork_processing_modules
import numpy as np
import os
import subprocess
import time

###------------------------------------------------------------------------------------###
###------------------------------------------------------------------------------------###

#PROGRAM THAT INTERACTIVELY RUNS DESIRED STAGES OF THE GHOST PIPELINE

###------------------------------------------------------------------------------------###
#SUMMARY OF STAGES:
###------------------------------------------------------------------------------------###
#N1 = Raw network data parse
#N2 = Resolve duplicate stations
#N3 = Parse globally gridded metadata
#N4 = Quality control
#N5 = Process classifications

#M1 = Process static modal metadata .csv

#T1 = Temporally grid by station
#T2 = Group temporally gridded data to monthly NETCDFs
###------------------------------------------------------------------------------------###

#BEGIN PROCESS! 

###------------------------------------------------------------------------------------###
#INITIALISATION : 1. Read configuration file for the setup of the processing system
#                 2. Iterate by network, by species
#                 3. Start timing clock for network/species combination
#                 4. Determine which stages of processing are wished to be done - taken from command line arguments 
###------------------------------------------------------------------------------------###

#Read configuration file
from GHOST_configuration import networks, parameters, start_year, end_year, n_processes, stages_to_process, ghost_version

#set number of processes to run in parallel
#1 Master + N Slave processes 
n_processes = n_processes+1

#Iterate through networks
for network in networks:

    #Iterate through parameters
    for parameter in parameters:
    
        #Process configuration variables to be read by all callable modules/submodules --> save data in configuration.npz file
        run_details = {'network':network,'parameter':parameter,'start_year':start_year,'end_year':end_year,'ghost_version':ghost_version}
        parameter_details = crossnetwork_processing_modules.get_parameter_details()[run_details['parameter']]
        network_details = crossnetwork_processing_modules.get_network_details()[run_details['network']]
        np.savez_compressed('GHOST_configuration.npz', run_details=run_details, parameter_details=parameter_details, network_details=network_details)

        #Start timer
        print 'BEGINNING PROCESSING OF ALL DESIRED STAGES FOR %s:%s'%(network,parameter)
        t0 = time.time()

###------------------------------------------------------------------------------------###
#STAGE N1 : 1. Process raw network files for a species
#           2. Outputs files to /native/component/temporary directory as an .npz per station 
###------------------------------------------------------------------------------------###
    
        if 'N1' in stages_to_process:
            crossnetwork_processing_modules.run_mpi4py_interactively(n_processes, 'N1', t0)

###------------------------------------------------------------------------------------###
#STAGE N2 : 1. Ensure /native/component/temporary/*.npz files are station specific 
###------------------------------------------------------------------------------------###

        if 'N2' in stages_to_process:
            crossnetwork_processing_modules.run_mpi4py_interactively(n_processes, 'N2', t0)

###------------------------------------------------------------------------------------###
#STAGE N3 : 1.
###------------------------------------------------------------------------------------###
    
###------------------------------------------------------------------------------------###
#STAGE N4 : 1. Do Quality Control per station 
#           2. Outputs to /native/component/ directory as an .npz per station
###------------------------------------------------------------------------------------###
        
        if 'N4' in stages_to_process:
            crossnetwork_processing_modules.run_mpi4py_interactively(n_processes, 'N4', t0)

###------------------------------------------------------------------------------------###
#STAGE N5 : 1. Do Classification checks per station
#           2. Rewrites .npz per station to /native/component/ directory
###------------------------------------------------------------------------------------###
      
        if 'N5' in stages_to_process:
            crossnetwork_processing_modules.run_mpi4py_interactively(n_processes, 'N5', t0)
            
###------------------------------------------------------------------------------------###
#STAGE M1 : 1. Calculate modal metadata per station across entire time record
#           2. Write to /metadata/processed .csv file
###------------------------------------------------------------------------------------###
      
        #if 'M1' in stages_to_process:
        #    crossnetwork_processing_modules.run_mpi4py_interactively(n_processes, 'M1', t0)
        
###------------------------------------------------------------------------------------###
#FINALISE PROCESS : 1. Stop timer 
###------------------------------------------------------------------------------------###

        #Stop timer
        t1 = time.time()
        total = t1-t0
        print 'PROCESSING OF ALL DESIRED STAGES FOR %s:%s COMPLETE IN %s MINUTES'%(network,parameter,total/60.)
        
###------------------------------------------------------------------------------------###
###------------------------------------------------------------------------------------###